@@ -444,6 +444,11 @@ const User& TASServer::GetMe() const {
     return GetUser( m_user );
 }
 
+wxString Useragent()
+{
+	return sett().Get( _T("Useragent"), GetAppName() ) + _T(" ") +
+		sett().Get( _T("Userversion"), GetSpringLobbyVersion(false) );
+}
 
 void TASServer::Login()
 {
@@ -454,8 +459,8 @@ void TASServer::Login()
 	localaddr = m_sock->GetLocalAddress();
     if ( localaddr.IsEmpty() ) localaddr = _T("*");
 	m_id_transmission = false;
-    SendCmd ( _T("LOGIN"), m_user + _T(" ") + pass + _T(" ") +
-			  GetHostCPUSpeed() + _T(" ") + localaddr + _T(" SpringLobby ") + GetSpringLobbyVersion(false) + protocol  + _T("\ta sp"));
+    wxFormat login_cmd( _T("%s %s %s %s %s\t%s\ta m sp") );
+    SendCmd ( _T("LOGIN"), (login_cmd % m_user % pass % GetHostCPUSpeed() % localaddr % Useragent() % protocol).str() );
 	m_id_transmission = true;
 }
 
@@ -1176,6 +1181,12 @@ void TASServer::ExecuteCommand( const wxString& cmd, const wxString& inparams, i
 				wxString description = GetSentenceParam( params );
 				m_se->OnFileDownload( parsingdata.tasdata.autoopen, parsingdata.tasdata.closelobbyondownload, parsingdata.tasdata.disconnectonrefuse, FileName, url, description );
     }
+    else if ( cmd == _T("FORCEJOINBATTLE") )
+    {
+        const int battleID = GetIntParam( params );
+        const wxString scriptpw = GetWordParam( params );
+        m_se->OnForceJoinBattle( battleID, scriptpw );
+    }
     else
     {
         wxLogMessage( _T("??? Cmd: %s params: %s"), cmd.c_str(), params.c_str() );
@@ -2320,19 +2331,19 @@ void TASServer::OnDisconnected( Socket* /*unused*/ )
 
 void TASServer::OnDataReceived( Socket* sock )
 {
-		if ( sock == 0 ) return;
-		m_last_net_packet = time( 0 );
+    if ( sock == 0 ) return;
+    m_last_net_packet = time( 0 );
     wxString data = sock->Receive();
-		m_buffer << data;
-		m_buffer.Replace( _T("\r\n"), _T("\n") );
-		int returnpos = m_buffer.Find( _T("\n") );
-		while ( returnpos != -1 )
-		{
-			wxString cmd = m_buffer.Left( returnpos );
-			m_buffer = m_buffer.Mid( returnpos + 1 );
-			ExecuteCommand( cmd );
-			returnpos = m_buffer.Find( _T("\n") );
-		}
+    m_buffer << data;
+    m_buffer.Replace( _T("\r\n"), _T("\n") );
+    int returnpos = m_buffer.Find( _T("\n") );
+    while ( returnpos != -1 )
+    {
+        wxString cmd = m_buffer.Left( returnpos );
+        m_buffer = m_buffer.Mid( returnpos + 1 );
+        ExecuteCommand( cmd );
+        returnpos = m_buffer.Find( _T("\n") );
+    }
 }
 
 
